package com.huigou.topsun.syncOrg.application.impl;

import cn.hutool.core.bean.BeanUtil;
import com.huigou.data.query.model.QueryDescriptor;
import com.huigou.topsun.common.OrgKindId;
import com.huigou.topsun.sap.common.DefaultHttpClient;
import com.huigou.topsun.syncOrg.application.OrgSyncApplication;
import com.huigou.topsun.syncOrg.vo.*;
import com.huigou.uasp.bmp.common.application.BaseApplication;
import com.huigou.uasp.bmp.opm.domain.model.org.Org;
import com.huigou.uasp.bmp.opm.domain.model.org.Person;
import com.huigou.uasp.bmp.opm.repository.org.OrgRepository;
import com.huigou.util.DateUtil;
import com.huigou.util.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Auther: xin.lu
 * @Date: 2024/03/25/9:28
 * @Description:
 */
@Service("orgSyncApplication")
public class OrgSyncApplicationImpl extends BaseApplication implements OrgSyncApplication {
    private final static Logger LOG = LoggerFactory.getLogger(OrgSyncApplicationImpl.class);
    @Autowired
    private OrgRepository orgRepository;
    @Autowired
    private DefaultHttpClient httpClient;

    @Override
    public EpOrgVo buildOrgVo () {
        List<Org> orgList = orgRepository.findAll();
        orgList = orgList
                .stream()
                .filter(org -> !"INIT_ORG".equals(org.getOrgCode()))
                .collect(Collectors.toList());
        //机构、部门、岗位
        List<OrgVo> orgVoList = orgList
                .stream()
                .filter(org -> !OrgKindId.PSM.getId().equals(org.getOrgKindId()))
                .map(org -> {
                    OrgVo orgVo = new OrgVo();
                    BeanUtil.copyProperties(org,orgVo);
                    orgVo.setDelFlag("0");
                    //把id做hash处理，得到int类型数据
                    orgVo.setId(convertHashCode(org.getId()));

                    //从全路径中获取当前组织的父节点名称
                    if (StringUtil.isNotBlank(org.getParentId())){
                        String fullName = org.getFullName();
                        String[] split = fullName.split("/");
                        orgVo.setParentName(split[split.length - 2]);

                        //把父id做hash处理，得到int类型数据
                        orgVo.setParentId(convertHashCode(org.getParentId()));
                    }
                    return orgVo;
                })
                .collect(Collectors.toList());

        //人员
        List<Org> psmList = orgList
                .stream()
                .filter(org -> OrgKindId.PSM.getId().equals(org.getOrgKindId()))
                .collect(Collectors.toList());

        //先同步机构、部门、岗位、再同步组织人员
        List<OrgPersonVo> orgPersonVoList = this.buildOrgPersonVo(psmList);

        //同步人员
        List<PersonVo> personVoList = this.buildPersonVo(psmList);

        EpOrgVo epOrgVo = new EpOrgVo();
        epOrgVo.setOrgVoList(orgVoList);
        epOrgVo.setOrgPersonVoList(orgPersonVoList);
        epOrgVo.setPersonVoList(personVoList);
        LOG.info(DateUtil.getDateFormat(new Date())+"====同步的组织结构数据===={}",epOrgVo);
        return epOrgVo;
    }

    @Override
    public List<OrgPersonVo> buildOrgPersonVo(List<Org> psmList){
        return psmList
                .stream()
                .map(orgPerson -> {
                    OrgPersonVo orgPersonVo = new OrgPersonVo();
                    //组织人员id
                    orgPersonVo.setId(convertHashCode(orgPerson.getId()));
                    //部门id
                    orgPersonVo.setDeptId(convertHashCode(orgPerson.getDeptId()));
                    //岗位id
                    orgPersonVo.setPositionId(convertHashCode(orgPerson.getPositionId()));
                    //人员id
                    orgPersonVo.setPersonId(convertHashCode(orgPerson.getPerson().getId()));
                    return orgPersonVo;
                }).collect(Collectors.toList());
    }

    @Override
    public List<PersonVo> buildPersonVo(List<Org> psmList){
        return psmList
                .stream()
                .map(psm -> {
                    Person person = psm.getPerson();
                    PersonVo personVo = new PersonVo();
                    BeanUtil.copyProperties(person, personVo);
                    personVo.setDelFlag("0");
                    //把人员id做hash处理，得到int类型数据
                    personVo.setId(convertHashCode(person.getId()));
                    personVo.setDeptId(convertHashCode(psm.getDeptId()));
                    return personVo;
                }).collect(Collectors.toList());
    }

    @Override
    public void syncOrgToSap() throws IOException {
        List<SapPersonVo> sapPersonVoList = this.slicedSapPersonVoList();
        String execute = httpClient.execute(sapPersonVoList, "");
        LOG.info("====同步到SAP返回消息===={}",execute);
        LOG.info(DateUtil.getDateFormat(new Date())+"====同步的组织结构数据===={}",sapPersonVoList);
    }

    @Override
    public List<SapPersonVo> slicedSapPersonVoList() {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "slicedSapPersonVoList");
        return this.sqlExecutorDao.queryToList(queryDescriptor.getSql(), SapPersonVo.class);
    }

    /**
     * 字符串转hashcode
     * @param value
     * @return
     */
    public String convertHashCode(String value){
        if (StringUtil.isBlank(value)){
            return "";
        }
        int abs = Math.abs(value.hashCode());
        return String.valueOf(abs);
    }
}
